from django.template.loader import select_template
from django.test import TestCase

from cms.api import add_plugin, create_page

from djangocms_bootstrap4.contrib.bootstrap4_carousel.cms_plugins import (
    Bootstrap4CarouselPlugin,
)
from djangocms_bootstrap4.contrib.bootstrap4_carousel.constants import (
    CAROUSEL_TEMPLATE_CHOICES,
)
from djangocms_bootstrap4.helpers import (
    concat_classes, get_plugin_template, get_template_path,
)


class B4HelpersTestCase(TestCase):

    def test_concat_classes(self):
        classes = ['class1 class2', 'class3']
        self.assertEqual(
            concat_classes(classes),
            'class1 class2 class3',
        )
        classes = []
        self.assertEqual(concat_classes(classes), '')

    def test_get_template_path(self):
        template = get_template_path(
            "carousel", "default", "slide"
        )
        result = "djangocms_bootstrap4/carousel/default/slide.html"
        self.assertEqual(template, result)
        status = select_template([template])
        self.assertEqual(status.template.name, result)

    def test_get_plugin_template(self):
        page = create_page(
            title="home",
            template="page.html",
            language="en",
        )
        instance = add_plugin(
            placeholder=page.placeholders.get(slot="content"),
            plugin_type=Bootstrap4CarouselPlugin.__name__,
            language="en",
        )
        template = get_plugin_template(
            instance, "carousel", "carousel",
            CAROUSEL_TEMPLATE_CHOICES,
        )
        self.assertEqual(template, "djangocms_bootstrap4/carousel/default/carousel.html")
        # trigger default template
        template = get_plugin_template(
            instance, "does_not", "exist",
            CAROUSEL_TEMPLATE_CHOICES,
        )
        self.assertEqual(template, "djangocms_bootstrap4/does_not/default/exist.html")
        # cleanup
        page.delete()



from cms.api import create_page


class B4TestFixture:
    """Sets up generic setUp and tearDown methods for tests."""

    def setUp(self):
        self.language = "en"
        self.home = create_page(
            title="home",
            template="page.html",
            language=self.language,
        )
        self.home.publish(self.language)
        self.page = create_page(
            title="content",
            template="page.html",
            language=self.language,
        )
        self.page.publish(self.language)
        self.placeholder = self.page.placeholders.get(slot="content")
        self.superuser = self.get_superuser()
        self.request_url = self.page.get_absolute_url(self.language) + "?toolbar_off=true"

        return super().setUp()

    def tearDown(self):
        self.page.delete()
        self.home.delete()
        self.superuser.delete()

        return super().tearDown()



from django.test import TestCase

from djangocms_bootstrap4.fields import (
    AttributesField, IntegerRangeField, TagTypeField,
)


class B4FieldsTestCase(TestCase):

    def test_attributes_field(self):
        field = AttributesField()
        self.assertEqual(field.verbose_name, "Attributes")
        self.assertEqual(field.blank, True)

    def test_tag_type_field(self):
        field = TagTypeField()
        self.assertEqual(field.verbose_name, "Tag type")
        self.assertEqual(field.choices, (
            ('div', 'div'),
            ('section', 'section'),
            ('article', 'article'),
            ('header', 'header'),
            ('footer', 'footer'),
            ('aside', 'aside')
        ))
        self.assertEqual(field.default, "div")
        self.assertEqual(field.max_length, 255)
        self.assertEqual(
            field.help_text,
            "Select the HTML tag to be used.",
        )

    def test_integer_range_field(self):
        field = IntegerRangeField()
        self.assertEqual(field.min_value, None)
        self.assertEqual(field.max_value, None)
        field.min_value = 255
        field.max_value = 255
        field = field.formfield()
        self.assertEqual(field.min_value, 255)
        self.assertEqual(field.max_value, 255)



from django.utils.translation import gettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_bootstrap4.helpers import concat_classes

from .models import Bootstrap4Badge


class Bootstrap4BadgePlugin(CMSPluginBase):
    """
    Components > "Badge" Plugin
    https://getbootstrap.com/docs/4.0/components/badge/
    """
    model = Bootstrap4Badge
    name = _('Badge')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/badge.html'
    change_form_template = 'djangocms_bootstrap4/admin/badge.html'
    text_enabled = True

    fieldsets = [
        (None, {
            'fields': (
                'badge_text',
                'badge_context',
                'badge_pills',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        link_classes = ['badge']
        if instance.badge_pills:
            link_classes.append('badge-pill')
        link_classes.append('badge-{}'.format(instance.badge_context))

        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super().render(
            context, instance, placeholder
        )


plugin_pool.register_plugin(Bootstrap4BadgePlugin)



# Generated by Django 1.9.13 on 2017-10-23 17:01
import django.db.models.deletion
from django.db import migrations, models

import djangocms_bootstrap4.fields
from djangocms_bootstrap4.constants import TAG_CHOICES


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Media',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_media_bootstrap4media', serialize=False, to='cms.CMSPlugin')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Bootstrap4MediaBody',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_media_bootstrap4mediabody', serialize=False, to='cms.CMSPlugin')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



#!/usr/bin/env python
HELPER_SETTINGS = {
    'INSTALLED_APPS': [
        'easy_thumbnails',
        'filer',
        'mptt',
        'djangocms_text_ckeditor',
        'djangocms_link',
        'djangocms_picture',
        'djangocms_bootstrap4',
        'djangocms_bootstrap4.contrib.bootstrap4_alerts',
        'djangocms_bootstrap4.contrib.bootstrap4_badge',
        'djangocms_bootstrap4.contrib.bootstrap4_card',
        'djangocms_bootstrap4.contrib.bootstrap4_carousel',
        'djangocms_bootstrap4.contrib.bootstrap4_collapse',
        'djangocms_bootstrap4.contrib.bootstrap4_content',
        'djangocms_bootstrap4.contrib.bootstrap4_grid',
        'djangocms_bootstrap4.contrib.bootstrap4_jumbotron',
        'djangocms_bootstrap4.contrib.bootstrap4_link',
        'djangocms_bootstrap4.contrib.bootstrap4_listgroup',
        'djangocms_bootstrap4.contrib.bootstrap4_media',
        'djangocms_bootstrap4.contrib.bootstrap4_picture',
        'djangocms_bootstrap4.contrib.bootstrap4_tabs',
        'djangocms_bootstrap4.contrib.bootstrap4_utilities',
    ],
    'CMS_LANGUAGES': {
        1: [{
            'code': 'en',
            'name': 'English',
        }]
    },
    # required otherwise subject_location would throw an error in the template
    'THUMBNAIL_PROCESSORS': (
        'easy_thumbnails.processors.colorspace',
        'easy_thumbnails.processors.autocrop',
        'filer.thumbnail_processors.scale_and_crop_with_subject_location',
        'easy_thumbnails.processors.filters',
    ),
    'LANGUAGE_CODE': 'en',
    'ALLOWED_HOSTS': ['localhost'],
    'DJANGOCMS_PICTURE_RESPONSIVE_IMAGES': False,
    'DJANGOCMS_PICTURE_RESPONSIVE_IMAGES_VIEWPORT_BREAKPOINTS': [576, 768, 992],
}


def run():
    from app_helper import runner
    runner.cms('djangocms_bootstrap4')


if __name__ == '__main__':
    run()



from django.test import TestCase

from djangocms_bootstrap4.constants import (
    ALIGN_CHOICES, COLOR_STYLE_CHOICES, DEVICE_CHOICES, DEVICE_SIZES,
    TAG_CHOICES,
)


class B4ConstantsTestCase(TestCase):
    # make sure to update the documentation
    # when changing any values in these

    def test_constants(self):
        self.assertEquals(DEVICE_CHOICES, (
            ('xs', 'Extra small'),   # default <576px
            ('sm', 'Small'),         # default ≥576px
            ('md', 'Medium'),        # default ≥768px
            ('lg', 'Large'),         # default ≥992px
            ('xl', 'Extra large'),   # default ≥1200px
        ))
        self.assertEquals(DEVICE_SIZES, ('xs', 'sm', 'md', 'lg', 'xl'))
        self.assertEquals(TAG_CHOICES, (
            ('div', 'div'),
            ('section', 'section'),
            ('article', 'article'),
            ('header', 'header'),
            ('footer', 'footer'),
            ('aside', 'aside')
        ))
        self.assertEquals(COLOR_STYLE_CHOICES, (
            ('primary', 'Primary'),
            ('secondary', 'Secondary'),
            ('success', 'Success'),
            ('danger', 'Danger'),
            ('warning', 'Warning'),
            ('info', 'Info'),
            ('light', 'Light'),
            ('dark', 'Dark'),
        ))
        self.assertEquals(ALIGN_CHOICES, (
            ('text-left', 'Left'),
            ('text-center', 'Center'),
            ('text-right', 'Right'),
        ))



from django.db import models
from django.utils.translation import gettext_lazy as _

from cms.models import CMSPlugin

from djangocms_bootstrap4.constants import COLOR_STYLE_CHOICES
from djangocms_bootstrap4.fields import AttributesField, TagTypeField


class Bootstrap4Alerts(CMSPlugin):
    """
    Components > "Alerts" Plugin
    https://getbootstrap.com/docs/4.0/components/alerts/
    """
    alert_context = models.CharField(
        verbose_name=_('Context'),
        choices=COLOR_STYLE_CHOICES,
        default=COLOR_STYLE_CHOICES[0][0],
        max_length=255,
    )
    alert_dismissable = models.BooleanField(
        verbose_name=_('Dismissable'),
        default=False,
        help_text=_('Allows the alert to be closed.'),
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        return '({})'.format(self.alert_context)
